@@ -1,7 +1,7 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <iostream.h>
+#include <iostream>
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -12,6 +12,8 @@
 
 #include "calcdb.h"
 
+using namespace std;
+
 #define ITSZ sizeof(int)
 
 char input[300];       //input file name
